@@ -11,7 +11,7 @@ import (
 	"testing"
 	"time"
 
-	diskbufferreader "github.com/bill-rich/disk-buffer-reader"
+	diskbufferreader "github.com/trufflesecurity/disk-buffer-reader"
 	"github.com/stretchr/testify/assert"
 
 	logContext "github.com/trufflesecurity/trufflehog/v3/pkg/context"
